package com.dayou.brokerage.config;

import com.dayou.brokerage.interceptor.BrokerageInterceptor;
import com.dayou.brokerage.validator.BrokerageValidator;
import org.springframework.aop.aspectj.AspectJExpressionPointcut;
import org.springframework.aop.support.DefaultPointcutAdvisor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * 类说明：提成逻辑相关配置类
 *
 * @author: wucl
 * @since: 2023/3/3
 * created with IntelliJ IDEA.
 */
@Configuration
public class BrokerageConfig {

    private static final String allServiceImpl = "execution(* com.dayou.service.impl.*.*(..))";

    @Autowired
    private BrokerageInterceptor brokerageInterceptor;


    @Bean
    public BrokerageValidator brokerageValidator() {
        BrokerageValidator brokerageValidator = new BrokerageValidator();
        return brokerageValidator;
    }

    @Bean(initMethod = "loadRatioList")
    public LandBrokerageDataConfiguration landBrokerageDataConfiguration(){
        LandBrokerageDataConfiguration landBrokerageDataConfiguration = new LandBrokerageDataConfiguration();
        return landBrokerageDataConfiguration;
    }

    @Bean
    public DefaultPointcutAdvisor brokerageAdvisor(){
        DefaultPointcutAdvisor advisor = new DefaultPointcutAdvisor();
        AspectJExpressionPointcut pointcut = new AspectJExpressionPointcut();
        pointcut.setExpression(allServiceImpl);
        advisor.setPointcut(pointcut);
        advisor.setAdvice(brokerageInterceptor);
        return advisor;
    }


}
