package com.dayou.brokerage.handler;

import com.dayou.bo.BrokerageDetailBO;
import com.dayou.bo.LeaderRatioBO;
import com.dayou.brokerage.BrokerageCalculateSupport;
import com.dayou.brokerage.MarketerBrokerageCalculator;
import com.dayou.brokerage.config.LandBrokerageDataConfiguration;
import com.dayou.brokerage.constants.BrokerageMode;
import com.dayou.brokerage.constants.BrokerageRule;
import com.dayou.brokerage.constants.BrokerageState;
import com.dayou.entity.ItemBrokerageDetail;
import com.dayou.exception.BusinessException;
import com.google.common.collect.Sets;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 类说明：土地规划部市场人员提成计算处理器
 *
 * @author: wucl
 * @since: 2023/3/6
 * created with IntelliJ IDEA.
 */
@Component
@Slf4j
public class LandMarketerBrokerageHandler extends BrokerageCalculateSupport implements MarketerBrokerageCalculator {

    @Autowired
    private LandBrokerageDataConfiguration landBrokerageDataConfiguration;

    @Override
    public BrokerageDetailBO predictPersonalAmount(BrokerageDetailBO bo) {
        String mergeKey = bo.mergeKey();
        BigDecimal ratio = landBrokerageDataConfiguration.ratioList.get(mergeKey);
        bo.setRate(ratio);
        if (ratio !=null){
            checkItemBrokerageStatus(bo);
            try {
                bo.setPredictAmount(bo.getAmount().multiply(ratio));
                bo.setBrokerageMode(BrokerageMode.PERSONAL.getCode());
                bo.setActualAmount(actualAmount(bo.getPredictAmount()));
                bo.setMarketerAdvanceBrokerageAble(checkMarketerPredictBrokerage(bo));
                if (checkMarketerPredictBrokerage(bo)){
                    bo.setMarketerStatus(BrokerageState.TO_PREDICTING.getCode());
                    bo.setAheadAmount(aheadAmount(bo.getPredictAmount()));
                }
            } catch (BusinessException e) {
                log.info("项目id:[{}],客户经理id：[{}],[{}]",bo.getId(),bo.getUserId(),e.getMessage());
            }
        }else {
            log.info("项目id:[{}],合同金额：[{}],提成系数：[null],[未设置提成系数，无法计算客户经理提成]",bo.getId(),bo.getAmount());
        }
        return bo;

    }

    @Override
    public BigDecimal aheadAmount(BigDecimal predictAmount){
        BigDecimal aheadAmount = predictAmount.multiply(landBrokerageDataConfiguration.marketerBrokeragePercentage);
        return aheadAmount;
    }

    @Override
    public Set<ItemBrokerageDetail> predictTeamShareAmount(BrokerageDetailBO bo, Set<LeaderRatioBO> leaderRatioBOSet) {
        String mergeKey = bo.mergeKey();
        BigDecimal ratio = landBrokerageDataConfiguration.ratioList.get(mergeKey);
        Set<ItemBrokerageDetail> itemBrokerageDetails = Sets.newHashSet();
        if (ratio!=null){
            checkItemBrokerageStatus(bo);
            itemBrokerageDetails = leaderRatioBOSet.stream().map(x -> {
                ItemBrokerageDetail itemBrokerageDetail = new ItemBrokerageDetail();
                itemBrokerageDetail.setGeneralId(bo.getGeneralId());
                itemBrokerageDetail.setPredictAmount(bo.getAmount().multiply(ratio).multiply(x.getRatio()));
                itemBrokerageDetail.setUserId(x.getUserId());
                itemBrokerageDetail.setAheadAmount(BigDecimal.ZERO);
                itemBrokerageDetail.setActualAmount(actualAmount(itemBrokerageDetail.getPredictAmount()));
                itemBrokerageDetail.setAdvanceAmount(BigDecimal.ZERO);
                itemBrokerageDetail.setBrokerageMode(BrokerageMode.TEAM_SHARE.getCode());
                itemBrokerageDetail.setBrokerageRule(BrokerageRule.LAND_MARKETER_RULE.getCode());
                return itemBrokerageDetail;
            }).collect(Collectors.toSet());

        }else {
            log.info("项目id:[{}],合同金额：[{}],提成系数：[null],[未设置提成系数，无法计算客户经理领导提成]",bo.getId(),bo.getAmount());
        }
        return itemBrokerageDetails;
    }

    @Override
    public BigDecimal actualAmount(BigDecimal predictAmount) {
        //客户经理的提成不存在考核，即 预计提成金额 == 实际提成金额
        return predictAmount;
    }

}
