package com.dayou.brokerage.handler;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dayou.bo.BrokerageDetailBO;
import com.dayou.brokerage.BrokerageCalculateSupport;
import com.dayou.brokerage.NonMarketerMarketerBrokerageCalculator;
import com.dayou.brokerage.config.LandBrokerageDataConfiguration;
import com.dayou.brokerage.constants.BrokerageRule;
import com.dayou.common.BaseEntity;
import com.dayou.dto.ParticipantSettleDTO;
import com.dayou.dto.SalarySettleDTO;
import com.dayou.entity.ItemBrokerageDetail;
import com.dayou.entity.ItemStage;
import com.dayou.service.IItemBrokerageDetailService;
import com.dayou.service.IItemBrokerageSequenceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 类说明：土地规划部 非 市场人员提成计算处理器
 *
 * @author: wucl
 * @since: 2023/3/8
 * created with IntelliJ IDEA.
 */
@Component
@Slf4j
public class LandNonMarketerBrokerageHandler extends BrokerageCalculateSupport implements NonMarketerMarketerBrokerageCalculator {

    @Autowired
    private LandBrokerageDataConfiguration landBrokerageDataConfiguration;

    @Autowired
    private IItemBrokerageDetailService brokerageDetailService;

    @Autowired
    private IItemBrokerageSequenceService brokerageSequenceService;

    @Override
    public ItemBrokerageDetail predictAmount(BrokerageDetailBO brokerageDetailBO) {
        checkItemBrokerageStatus(brokerageDetailBO);
        ItemBrokerageDetail itemBrokerageDetail = new ItemBrokerageDetail();
        if (landBrokerageDataConfiguration.participantBrokeragePercentage!=null){
            //合同金额
            BigDecimal predictAmount = brokerageDetailBO.getAmount()
                    //参与项目提成总比例
                    .multiply(landBrokerageDataConfiguration.participantBrokeragePercentage)
                    //阶段权重
                    .multiply(brokerageDetailBO.getStageWeight())
                    //参与人权重
                    .multiply(brokerageDetailBO.getParticipantWeight());
            brokerageDetailBO.setPredictAmount(predictAmount.setScale(2,BigDecimal.ROUND_HALF_UP));
            BeanUtil.copyProperties(brokerageDetailBO,itemBrokerageDetail);
        }
        log.info("项目id:[{}],合同金额：[{}],提成系数：[null],[未设置[土规部参与项目整体提成比例]，无法计算参与人员提成]",brokerageDetailBO.getItemId(),brokerageDetailBO.getAmount());
        return itemBrokerageDetail;
    }

    @Override
    public BigDecimal aheadAmount(Long userId){
        BigDecimal baseAheadAmount = landBrokerageDataConfiguration.userBaseBrokerageAmounts.get(userId);
        return baseAheadAmount;
    }
    @Override
    public List<ItemBrokerageDetail> actualAmount(ParticipantSettleDTO participantSettleDTO) {
        checkItemBrokerageStatusOnParticipantSettle(participantSettleDTO.getBrokerageState());
        List<ItemBrokerageDetail> participantBrokerageDetails = brokerageDetailService.list(new LambdaQueryWrapper<ItemBrokerageDetail>().eq(ItemBrokerageDetail::getGeneralId, participantSettleDTO.getGeneralId())
                .eq(BaseEntity::getDeleted, Boolean.FALSE).eq(ItemBrokerageDetail::getBrokerageRule, BrokerageRule.LAND_OTHER_RULE.getCode()));
        Map<Long, List<ItemBrokerageDetail>> brokerageByStage = participantBrokerageDetails.stream().collect(Collectors.groupingBy(ItemBrokerageDetail::getBrokerageStageId));
        for (Map.Entry<Long,List<ItemBrokerageDetail>> detail : brokerageByStage.entrySet()){
            for (ItemStage stage : participantSettleDTO.getStages()){
                if (detail.getKey().equals(stage.getId())){
                    List<ItemBrokerageDetail> value = detail.getValue();
                    for (ItemBrokerageDetail brokerage :value){
                        brokerage.setActualAmount(brokerage.getPredictAmount().multiply(participantSettleDTO.getWeight()));
                        if (stage.getOverdue()){
                            brokerage.setActualAmount(brokerage.getActualAmount().multiply(stage.getCoefficient()));
                        }
                    }
                }
            }
        }

        return participantBrokerageDetails;
    }

    @Override
    public BigDecimal advanceTotalAmount() {
        return null;
    }

    @Override
    public int advanceTimes() {
        return 0;
    }
}
